

Imports System.Collections

Public Class ChannelMessageBuilder
    Implements IMessageBuilder

    Private Shared messageCache As Hashtable = Hashtable.Synchronized(New Hashtable())

    ' ---------------------------------------------------------- the channel message as a packed integer
    Private m_message As Integer = 0

    ' ---------------------------------------------------------- the built ChannelMessage
    Private m_result As ChannelMessage = Nothing


    Public Sub New()
        Command = ChannelCommand.Controller
        MidiChannel = 0
        Data1 = ControllerType.AllSoundOff
        Data2 = 0
    End Sub

    Public Sub New(ByVal message As ChannelMessage)
        Initialize(message)
    End Sub


    ' ---------------------------------------------------------------------------------------------- "Methods"

    Public Sub Initialize(ByVal message As ChannelMessage)
        Me.m_message = message.Message
    End Sub

    Public Shared Sub Clear()
        messageCache.Clear()
    End Sub


    ' --------------------------------------------------------------------------------------------- "Properties"

    Public Shared ReadOnly Property Count() As Integer
        Get
            Return messageCache.Count
        End Get
    End Property

    Public ReadOnly Property Result() As ChannelMessage
        Get
            Return m_result
        End Get
    End Property

    ' ---------------------------------------------- gets or sets the ChannelMessageEventArgs as a packed integer 
    Friend Property Message() As Integer
        Get
            Return m_message
        End Get
        Set(ByVal value As Integer)
            m_message = value
        End Set
    End Property

    ' ---------------------------------------------- gets or sets the Command value for building ChannelMessageEventArgs
    Public Property Command() As ChannelCommand
        Get
            Return ChannelMessage.UnpackCommand(m_message)
        End Get
        Set(ByVal value As ChannelCommand)
            m_message = ChannelMessage.PackCommand(m_message, value)
        End Set
    End Property

    ' ---------------------------------------------- gets or sets the MIDI channel for building the ChannelMessageEventArgs
    Public Property MidiChannel() As Integer
        Get
            Return ChannelMessage.UnpackMidiChannel(m_message)
        End Get
        Set(ByVal value As Integer)
            m_message = ChannelMessage.PackMidiChannel(m_message, value)
        End Set
    End Property

    ' ---------------------------------------------- gets or sets the first data value for building the ChannelMessageEventArgs
    Public Property Data1() As Integer
        Get
            Return ShortMessage.UnpackData1(m_message)
        End Get
        Set(ByVal value As Integer)
            m_message = ShortMessage.PackData1(m_message, value)
        End Set
    End Property

    ' ---------------------------------------------- gets or sets thr second data value for building the ChannelMessageEventArgs
    Public Property Data2() As Integer
        Get
            Return ShortMessage.UnpackData2(m_message)
        End Get
        Set(ByVal value As Integer)
            m_message = ShortMessage.PackData2(m_message, value)
        End Set
    End Property


    Public Sub Build() Implements IMessageBuilder.Build
        m_result = DirectCast(messageCache(m_message), ChannelMessage)
        ' ------------------------------------------------------------- if the message does not exist
        If m_result Is Nothing Then
            m_result = New ChannelMessage(m_message)
            ' --------------------------------------------------------- add message to cache
            messageCache.Add(m_message, m_result)
        End If
    End Sub

End Class
