

Imports System.Collections

Public Class ChannelStopper

    Private noteOnMessage As ChannelMessage(,)
    Private holdPedal1Message As Boolean()
    Private holdPedal2Message As Boolean()
    Private sustenutoMessage As Boolean()
    Private builder As New ChannelMessageBuilder()
    Public Event Stopped As EventHandler(Of StoppedEventArgs)

    Public Sub New()
        Dim c As Integer = ChannelMessage.MidiChannelMaxValue + 1
        Dim d As Integer = ShortMessage.DataMaxValue + 1
        noteOnMessage = New ChannelMessage(c - 1, d - 1) {}
        holdPedal1Message = New Boolean(c - 1) {}
        holdPedal2Message = New Boolean(c - 1) {}
        sustenutoMessage = New Boolean(c - 1) {}
    End Sub

    Public Sub Process(ByVal message As ChannelMessage)
        Select Case message.Command
            Case ChannelCommand.NoteOn
                If message.Data2 > 0 Then
                    noteOnMessage(message.MidiChannel, message.Data1) = message
                Else
                    noteOnMessage(message.MidiChannel, message.Data1) = Nothing
                End If

            Case ChannelCommand.NoteOff
                noteOnMessage(message.MidiChannel, message.Data1) = Nothing

            Case ChannelCommand.Controller
                Select Case message.Data1
                    Case ControllerType.HoldPedal1
                        If message.Data2 > 63 Then
                            holdPedal1Message(message.MidiChannel) = True
                        Else
                            holdPedal1Message(message.MidiChannel) = False
                        End If

                    Case ControllerType.HoldPedal2
                        If message.Data2 > 63 Then
                            holdPedal2Message(message.MidiChannel) = True
                        Else
                            holdPedal2Message(message.MidiChannel) = False
                        End If

                    Case ControllerType.SustenutoPedal
                        If message.Data2 > 63 Then
                            sustenutoMessage(message.MidiChannel) = True
                        Else
                            sustenutoMessage(message.MidiChannel) = False
                        End If

                End Select

        End Select
    End Sub

    Public Sub AllSoundOff()
        Dim stoppedMessages As New ArrayList()

        For c As Integer = 0 To ChannelMessage.MidiChannelMaxValue
            For n As Integer = 0 To ShortMessage.DataMaxValue
                If noteOnMessage(c, n) IsNot Nothing Then
                    builder.MidiChannel = c
                    builder.Command = ChannelCommand.NoteOff
                    builder.Data1 = noteOnMessage(c, n).Data1
                    builder.Build()

                    stoppedMessages.Add(builder.Result)

                    noteOnMessage(c, n) = Nothing
                End If
            Next

            If holdPedal1Message(c) Then
                builder.MidiChannel = c
                builder.Command = ChannelCommand.Controller
                builder.Data1 = ControllerType.HoldPedal1
                builder.Build()

                stoppedMessages.Add(builder.Result)

                holdPedal1Message(c) = False
            End If

            If holdPedal2Message(c) Then
                builder.MidiChannel = c
                builder.Command = ChannelCommand.Controller
                builder.Data1 = ControllerType.HoldPedal2
                builder.Build()

                stoppedMessages.Add(builder.Result)

                holdPedal2Message(c) = False
            End If

            If sustenutoMessage(c) Then
                builder.MidiChannel = c
                builder.Command = ChannelCommand.Controller
                builder.Data1 = ControllerType.SustenutoPedal
                builder.Build()

                stoppedMessages.Add(builder.Result)

                sustenutoMessage(c) = False
            End If
        Next

        OnStopped(New StoppedEventArgs(stoppedMessages))
    End Sub

    Public Sub Reset()
        For c As Integer = 0 To ChannelMessage.MidiChannelMaxValue
            For n As Integer = 0 To ShortMessage.DataMaxValue
                noteOnMessage(c, n) = Nothing
            Next

            holdPedal1Message(c) = False
            holdPedal2Message(c) = False
            sustenutoMessage(c) = False
        Next
    End Sub

    Protected Overridable Sub OnStopped(ByVal e As StoppedEventArgs)
        RaiseEvent Stopped(Me, e)
    End Sub
End Class
