

Imports System.Diagnostics

' -------------------------------------------------------------- functionality for choosing Oscillator waveform
Public Class OscWaveProgrammer
	Implements IProgrammable

    Public Enum ParameterId
        WaveformType
    End Enum

    Public Enum WaveformType
        Sawtooth
        Square
        Triangle
        Sine
        Bell
        Organ
        Harpsichord
        Bass
    End Enum

    Private Const WaveformTypeCount As Integer = WaveformType.Bass + 1
    Private name As String = ""
    Private osc As Oscillator
    Private type As WaveformType

    Public Sub New(ByVal name As String, ByVal osc As Oscillator)
        If osc Is Nothing Then
            Throw New ArgumentNullException("osc")
        End If
        Me.name = name
        Me.osc = osc
        SetParameterValue(0, 0)
    End Sub


#Region "IProgrammable Members"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As String = String.Empty
        '
        Dim s As String = name
        If Not String.IsNullOrEmpty(s) Then
            s = s & " "
        End If
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.WaveformType
                result = s & "Waveform"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If

        Dim result As String = String.Empty

        Select Case CType(index, ParameterId)
            Case ParameterId.WaveformType
                result = "Type"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select

        Return result
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If

        Dim result As String = String.Empty

        Select Case CType(index, ParameterId)
            Case ParameterId.WaveformType
                result = type.ToString()
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select

        Return result
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If

        Dim result As Single = 0

        Select Case CType(index, ParameterId)
            Case ParameterId.WaveformType
                result = CSng(type / (WaveformTypeCount - 1))
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select

        Return result
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If

        Select Case CType(index, ParameterId)
            Case ParameterId.WaveformType
                type = CType((WaveformTypeCount - 1) * value, WaveformType)
                osc.Wavetable = Wavetable.Load(type.ToString())
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return ParameterId.WaveformType + 1
        End Get
    End Property

#End Region

End Class
