

Imports System.IO

' ------------------------------------------------------------------------- a synthesizer program or "patch" 
Friend NotInheritable Class Program

    Private Const ProgramFileHeader As String = "Synth Program File"
    Private Const ProgramNameDefaultValue As String = "Empty"
    Private values As Single()
    Private synthName As String
    Private programName As String


    Public Sub New(ByVal synthName As String, ByVal parameterCount As Integer)

        If synthName Is Nothing Then
            Throw New ArgumentNullException("synthName")
        ElseIf parameterCount < 0 Then
            Throw New ArgumentOutOfRangeException("parameterCount")
        End If

        Me.synthName = synthName
        Me.programName = ProgramNameDefaultValue

        values = New Single(parameterCount - 1) {}
    End Sub

 
    Public Sub New(ByVal synthName As String, ByVal programName As String, ByVal parameterCount As Integer)

        If synthName Is Nothing Then
            Throw New ArgumentNullException("synthName")
        ElseIf programName Is Nothing Then
            Throw New ArgumentNullException("programName")
        ElseIf parameterCount < 0 Then
            Throw New ArgumentOutOfRangeException("parameterCount")
        End If

        Me.synthName = synthName
        Me.programName = programName

        values = New Single(parameterCount - 1) {}
    End Sub



    ' -------------------------------------------------- Loads data into Program using the specified file name
    Public Sub Load(ByVal fileName As String)
        Using stream As New FileStream(fileName, FileMode.Open, FileAccess.Read)
            Dim reader As New BinaryReader(stream)

            If reader.ReadString() <> ProgramFileHeader Then
                reader.Close()

                Throw New FileException("Wrong type of file.")
            End If

            If reader.ReadString() <> synthName Then
                reader.Close()

                Throw New FileException("Wrong type of file.")
            End If

            ' ------------------------------------------------------------ Read the parameter count.
            Dim p As Integer = reader.ReadInt32()

            ' ------------------------------------------------------------ Choose the smaller number of parameters. 
            '                                                              This gives some flexibility with changing
            '                                                              the number of parameters, though
            '                                                              mismatching parameters to values is possible
            p = (If(p < ParameterCount, p, ParameterCount))
            Dim newProgramName As String = reader.ReadString()
            Dim newValues As Single() = New Single(ParameterCount - 1) {}
            ' ------------------------------------------------------------ For every parameter value in the program file.
            For i As Integer = 0 To p - 1
                ' -------------------------------------------------------- Read parameter value.
                newValues(i) = reader.ReadSingle()
                If newValues(i) < 0 OrElse newValues(i) > 1 Then
                    Throw New FileException("Program file value out of range.")
                End If
            Next
            programName = newProgramName
            values = newValues
        End Using
    End Sub

    ' ---------------------------------------------------- Saves program data to file using the specified file name.
    Public Sub Save(ByVal fileName As String)
        Using stream As New FileStream(fileName, FileMode.Create)
            Dim writer As New BinaryWriter(stream)
            writer.Write(ProgramFileHeader)
            writer.Write(synthName)
            writer.Write(ParameterCount)
            writer.Write(programName)
            For i As Integer = 0 To ParameterCount - 1
                writer.Write(values(i))
            Next
            writer.Close()
        End Using
    End Sub


    ' -------------------------------------- Gets or sets the parameter value using the specified zero-based index.
    Default Public Property Item(ByVal index As Integer) As Single
        Get
            Return values(index)
        End Get
        Set(ByVal value As Single)
            '#Region "Require"
            If index < 0 OrElse index >= ParameterCount Then
                Throw New ArgumentOutOfRangeException("index")
            ElseIf value < 0 OrElse value > 1 Then
                Throw New ArgumentOutOfRangeException("value")
            End If
            '#End Region

            values(index) = value
        End Set
    End Property

    ' ------------------------------------------------------- Gets the number of parameters the Program represents
    Public ReadOnly Property ParameterCount() As Integer
        Get
            Return values.Length
        End Get
    End Property

    ' ------------------------------------------------------- Gets or sets the program name.
    Public Property Name() As String
        Get
            Return programName
        End Get
        Set(ByVal value As String)
            If value Is Nothing Then
                Throw New ArgumentNullException("Name")
            End If
            programName = value
        End Set
    End Property

End Class
