

Imports System.Diagnostics

Public Class SlewLimiter
	Inherits MonoSynthComponent
	Implements IProgrammable

    Public Enum ParameterId
        SlewTime
    End Enum

    Private Enum Direction
        Up
        Down
    End Enum

    Private Const Ceiling As Single = 0.63212F
    Private Const TimeScaler As Integer = 1

    Private slewTime As Single = 0.25F
    Private m_direction As Direction
    Private target As Single = 0
    Private a0 As Single, b1 As Single
    Private output As Single
    Private m_synthesizeReplaceEnabled As Boolean = True


    Public Sub New(ByVal sampleRate As SampleRate, ByVal buffer As MonoBuffer)
        MyBase.New(sampleRate, buffer)
    End Sub

    Public Sub New(ByVal sampleRate As SampleRate, ByVal buffer As MonoBuffer, ByVal name As String)
        MyBase.New(sampleRate, buffer, name)
    End Sub


    Public Overloads Overrides Sub Synthesize(ByVal offset As Integer, ByVal count As Integer)
        Dim buffer As Single() = GetBuffer()
        Dim c As Integer = offset + count
        Dim o As Single
        For i As Integer = offset To c - 1
            o = 0
            If m_direction = Direction.Up Then
                If output < target Then
                    o = a0 * target + b1 * output
                    output = o
                End If
            Else
                If output > target Then
                    o = a0 * target + b1 * output
                    output = o
                End If
            End If
            buffer(i) = (If(m_synthesizeReplaceEnabled, o, buffer(i) + o))
        Next
    End Sub

    Public Overloads Overrides Sub Trigger(ByVal previousNote As Integer, ByVal note As Integer, ByVal velocity As Single)
        Dim s As Single = slewTime * TimeScaler * SamplesPerSecond + 1
        Dim x As Single = CSng(Math.Exp(-1 / s))
        a0 = x - 1
        b1 = x
        output = CSng((previousNote - note)) / NotesPerOctave
        If output < 0 Then
            m_direction = Direction.Up
            target = 0.01F
        Else
            m_direction = Direction.Down
            target = -0.01F
        End If
    End Sub

    Public Overloads Overrides Sub Release(ByVal velocity As Single)
    End Sub


    Public Overloads Overrides ReadOnly Property Ordinal() As Integer
        Get
            Return 1
        End Get
    End Property

    Public Overloads Overrides Property SynthesizeReplaceEnabled() As Boolean
        Get
            Return m_synthesizeReplaceEnabled
        End Get
        Set(ByVal value As Boolean)
            m_synthesizeReplaceEnabled = value
        End Set
    End Property



#Region "IProgrammable Members"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If

        Dim result As String = ""
        Dim name__1 As String = Name
        If Not String.IsNullOrEmpty(name__1) Then name__1 = name__1 & " "

        Select Case CType(index, ParameterId)
            Case ParameterId.SlewTime : result = name__1 & "Time"
            Case Else : Debug.Fail("Unhandled parameter")
        End Select

        Return result
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.SlewTime : result = "Time"
            Case Else : Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = String.Empty
        Select Case CType(index, ParameterId)
            Case ParameterId.SlewTime
                If True Then
                    Dim seconds As Single = slewTime * TimeScaler * SamplesPerSecond / SamplesPerSecond
                    result = seconds.ToString(Synthesizer.FormatInfo)
                End If
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As Single = 0
        Select Case CType(index, ParameterId)
            Case ParameterId.SlewTime
                result = slewTime
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        ElseIf value < 0 OrElse value > 1 Then
            Throw New ArgumentOutOfRangeException("value")
        End If
        Select Case CType(index, ParameterId)
            Case ParameterId.SlewTime
                slewTime = value
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return ParameterId.SlewTime + 1
        End Get
    End Property

#End Region

End Class
