

Imports System.Collections.Generic
Imports LC.Midi

' ------------------------------------------------------------------------ processing MIDI control messages.
Friend Class ControlProcessor

    Private voices As List(Of Voice)

    Private allocator As VoiceAllocator

    Public Sub New(ByVal voices As IEnumerable(Of Voice), ByVal allocator As VoiceAllocator)
        Me.voices = New List(Of Voice)(voices)
        Me.allocator = allocator
    End Sub


    ' -------------------------------------------------------------------- Processes a control message
    Public Sub Process(ByVal type As ControllerType, ByVal value As Integer)

        ' ---------------------------------------------------------------- hold pedal 1 (sustain pedal) message.
        If type = ControllerType.HoldPedal1 Then

            If value < 64 Then
                ' -------------------------------------------------------- the sustain pedal is turned off
                allocator.SustainEnabled = False
            Else
                ' -------------------------------------------------------- the sustain pedal is turned on
                allocator.SustainEnabled = True
            End If

            ' ------------------------------------------------------------ all sound off message.
        ElseIf type = ControllerType.AllSoundOff Then
            allocator.AllSoundOff()

            ' ------------------------------------------------------------ some other type of control message.
        Else
            ' ------------------------------------------------------------ convert control message to float [0,1]
            Dim fValue As Single = CSng(value) / ChannelMessage.DataMaxValue
            ' ------------------------------------------------------------ for each Voice, pass the control message
            For Each v As Voice In voices
                v.ProcessControllerMessage(type, fValue)
            Next
        End If
    End Sub

End Class
