

Imports System.Collections.Generic
Imports System.Diagnostics

' --------------------------------------------------------------------- taking and converting pitch bend message data 
Friend Class PitchBender
    Implements IProgrammable

    Public Enum ParameterId
        PitchBendRange
    End Enum


    Public Const PitchBendRangeMaxValue As Integer = 12
    Private Const PitchBendRangeCenterValue As Integer = 8192
    Private pitchBendRange As Single = 2

    ' ------------------------------------------------------------------ the Voices that will receive pitch bend data
    Private voices As List(Of Voice)



    Public Sub New(ByVal voices As IEnumerable(Of Voice))
        Me.voices = New List(Of Voice)(voices)
    End Sub


    ' ------------------------------------------------------------------- Takes pitch bend message data,
    '                                                                     converts it to a floating point number
    '                                                                     in the range of [-1, 1], 
    '                                                                     and sends it to its Voice collection.
    Public Sub BendPitch(ByVal data1 As Integer, ByVal data2 As Integer)
        ' --------------------------------------------------------------- Pack pitch bend values.
        Dim pitchBendModulation As Single = data1 Or (data2 << 7)
        ' --------------------------------------------------------------- Offset pitch bend modulation.
        pitchBendModulation -= PitchBendRangeCenterValue
        ' --------------------------------------------------------------- pitch bend modulation to a float [-1, 1]
        pitchBendModulation /= PitchBendRangeCenterValue
        ' --------------------------------------------------------------- scale by pitch bend range parameter
        pitchBendModulation *= (pitchBendRange / PitchBendRangeMaxValue)
        ' --------------------------------------------------------------- initialize for all Voices
        For Each voice As Voice In voices
            voice.PitchBendModulation = pitchBendModulation
        Next
    End Sub


#Region "IProgrammable Members"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.PitchBendRange
                result = "Pitch Bend Range"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.PitchBendRange
                result = "Semitones"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.PitchBendRange
                result = pitchBendRange.ToString(Synthesizer.FormatInfo)
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As Single = 0
        Select Case CType(index, ParameterId)
            Case ParameterId.PitchBendRange
                result = pitchBendRange / PitchBendRangeMaxValue
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        ElseIf value < 0 OrElse value > 1 Then
            Throw New ArgumentOutOfRangeException("value")
        End If
        Select Case CType(index, ParameterId)
            Case ParameterId.PitchBendRange
                pitchBendRange = PitchBendRangeMaxValue * value
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return ParameterId.PitchBendRange + 1
        End Get
    End Property

#End Region

End Class
