

Imports System.Collections.Generic


' ----------------------------------------------------------------------- parameters from a collection of Voices
'
' The VoiceParameters class provides functionality for getting and setting
' parameters for a collection of Voices. This functionality makes it 
' convenient when setting Voice parameters because you don't have to
' iterate through the collection of Voices in order to set their 
' parameters yourself. The VoiceParameter class takes care of it for you. 
'
Friend Class VoiceParameters
	Implements IProgrammable

    ' ------------------------------------------ The collection of Voices this VoiceParameters provides access to.
    Private voices As List(Of Voice)

    ' ------------------------------------------ The number of parameters.
    Private m_parameterCount As Integer


    Public Sub New(ByVal voices As IEnumerable(Of Voice))
        Me.voices = New List(Of Voice)(voices)
        If Me.voices.Count > 0 Then
            m_parameterCount = Me.voices(0).ParameterCount
        End If
    End Sub



#Region "IProgrammable Members"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Return voices(0).GetParameterName(index)
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Return voices(0).GetParameterLabel(index)
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Return voices(0).GetParameterDisplay(index)
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Return voices(0).GetParameterValue(index)
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        For Each v As Voice In voices
            v.SetParameterValue(index, value)
        Next
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return m_parameterCount
        End Get
    End Property

#End Region

End Class
