

Imports System.Collections.Generic
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Threading
Imports System.Timers
Imports LC.Multimedia
Imports LC.Timers
Imports LC.Threading


Public Partial Class OutputDevice
    Inherits WaveDevice

    Private Const WOM_DONE As Integer = &H3BD

    Private Const WAVECAPS_PITCH As Integer = &H1           ' supports pitch control 
    Private Const WAVECAPS_PLAYBACKRATE As Integer = &H2    ' supports playback rate control 
    Private Const WAVECAPS_VOLUME As Integer = &H4          ' supports volume control 
    Private Const WAVECAPS_LRVOLUME As Integer = &H8        ' separate left-right volume control 

    Private Const WHDR_BEGINLOOP As Integer = &H4           ' loop start block 
    Private Const WHDR_ENDLOOP As Integer = &H8             ' loop end block 

    Private Const CALLBACK_NULL As Integer = &H0

	Private m_handle As Int32 = 0
	Private caps As New WaveOutCaps()
	Private asyncResults As New Queue(Of OutputAsyncResult)()
	Private timer As New System.Timers.Timer()

	Private ReadOnly lockObject As New Object()

	Public Sub New(deviceID As Integer, channelCount As Short, samplesPerSecond As Integer, bitsPerSecond As Short)
		MyBase.New(deviceID)
		NativeMethods.waveOutGetDevCaps(deviceID, caps, Marshal.SizeOf(GetType(WaveOutCaps)))

		Dim format As WaveFormat = CreateWaveFormat(channelCount, samplesPerSecond, bitsPerSecond)

		Dim result As Integer = NativeMethods.waveOutOpen(m_handle, deviceID, format, IntPtr.Zero, IntPtr.Zero, CALLBACK_NULL)

		If result <> DeviceException.MMSYSERR_NOERROR Then
            Return
        End If

		AddHandler timer.Elapsed, AddressOf ElapsedHandler

        timer.Start()
	End Sub

	Private Sub ElapsedHandler(sender As Object, e As ElapsedEventArgs)
		SyncLock lockObject
			If asyncResults.Count > 0 Then
				Dim asyncResult As OutputAsyncResult = asyncResults.Peek()
				Dim buffer As WaveformBuffer = asyncResult.Buffer

				If buffer.IsDone Then
					Dim result As Integer = NativeMethods.waveOutUnprepareHeader(m_handle, buffer.HeaderPointer, SizeOfWaveHeader)

					If result <> OutputDeviceException.MMSYSERR_NOERROR Then
						asyncResult.[Error] = New OutputDeviceException(result)
					End If

					asyncResult.Signal()
					asyncResults.Dequeue()

					Monitor.Pulse(lockObject)
				End If
			End If
		End SyncLock
    End Sub

	Protected Overrides Sub Finalize()
		Try

			Dispose(False)
		Finally
			MyBase.Finalize()
		End Try
	End Sub

	Protected Overloads Overrides Sub Dispose(disposing As Boolean)
		If disposing Then
			Reset()

			timer.Dispose()

			Dim result As Integer = NativeMethods.waveOutClose(m_handle)

			If result <> DeviceException.MMSYSERR_NOERROR Then
				Throw New OutputDeviceException(result)
			End If
		Else
			NativeMethods.waveOutReset(m_handle)
			NativeMethods.waveOutClose(m_handle)
		End If

		MyBase.Dispose(disposing)
	End Sub

	Public Overloads Overrides Sub Close()
        If IsDisposed Then
            Return
        End If
        Dispose()
    End Sub

	Public Overloads Overrides Sub Reset()
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        End If
        SyncLock lockObject
            Dim result As Integer = NativeMethods.waveOutReset(m_handle)

            If result = DeviceException.MMSYSERR_NOERROR Then
                While asyncResults.Count > 0
                    Monitor.Wait(lockObject)
                End While
            Else
                Throw New OutputDeviceException(result)
            End If
        End SyncLock
    End Sub

	Public Sub Pause()
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        End If
        SyncLock lockObject
            Dim result As Integer = NativeMethods.waveOutPause(m_handle)
            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If
        End SyncLock
    End Sub

	Public Sub Restart()
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        End If
        SyncLock lockObject
            Dim result As Integer = NativeMethods.waveOutRestart(m_handle)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If
        End SyncLock
    End Sub

	Public Function BeginSend(buffer As WaveformBuffer, callback As AsyncCallback, state As Object) As IAsyncResult
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        ElseIf buffer Is Nothing Then
            Throw New ArgumentNullException("wave")
        End If

        Dim asyncResult As New OutputAsyncResult(buffer, Me, callback, state)

        SyncLock lockObject
            buffer.CopyToMemory()

            Dim result As Integer = NativeMethods.waveOutPrepareHeader(m_handle, buffer.HeaderPointer, SizeOfWaveHeader)

            If result = DeviceException.MMSYSERR_NOERROR Then
                result = NativeMethods.waveOutWrite(m_handle, buffer.HeaderPointer, SizeOfWaveHeader)

                If result <> DeviceException.MMSYSERR_NOERROR Then
                    NativeMethods.waveOutUnprepareHeader(m_handle, buffer.HeaderPointer, SizeOfWaveHeader)

                    Throw New OutputDeviceException(result)
                End If
            Else
                Throw New OutputDeviceException(result)
            End If

            asyncResults.Enqueue(asyncResult)
        End SyncLock

        Return asyncResult
    End Function

	Public Sub EndSend(result As IAsyncResult)
		Dim r As OutputAsyncResult = TryCast(result, OutputAsyncResult)

		If r Is Nothing Then
			Throw New ArgumentException("Wrong type of IAsyncResult object.")
		ElseIf r.Owner IsNot Me Then
			Throw New ArgumentException("This IAsyncResult object did not originate on this instance.")
		End If

		r.AsyncWaitHandle.WaitOne()

		If r.[Error] IsNot Nothing Then
			Throw r.[Error]
		End If
	End Sub

	Public Sub Send(buffer As WaveformBuffer)
		EndSend(BeginSend(buffer, Nothing, Nothing))
    End Sub

    Public Function IsWorking() As Boolean
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        End If
        Dim t As New Time()
        t.type = TimeType.Milliseconds
        Dim result As Integer = NativeMethods.waveOutGetPosition(m_handle, t, Marshal.SizeOf(GetType(Time)))
        If result <> DeviceException.MMSYSERR_NOERROR Then
            Return False
        End If
        Return True
    End Function

	Public Function GetPosition(type As TimeType) As Time
        If IsDisposed Then
            Throw New ObjectDisposedException("OutputDevice")
        End If
        Dim t As New Time()
        t.type = type
        Dim result As Integer = NativeMethods.waveOutGetPosition(m_handle, t, Marshal.SizeOf(GetType(Time)))
        If result <> DeviceException.MMSYSERR_NOERROR Then
            Throw New OutputDeviceException(result)
        End If
        Return t
    End Function

	Public Shared Function GetDeviceCapabilities(deviceID As Integer) As WaveOutCaps
		Dim caps As New WaveOutCaps()
		Dim result As Integer = NativeMethods.waveOutGetDevCaps(deviceID, caps, Marshal.SizeOf(GetType(WaveOutCaps)))

		If result <> DeviceException.MMSYSERR_NOERROR Then
			Throw New OutputDeviceException(result)
		End If

		Return caps
	End Function

	Private Shared Function CreateWaveFormat(channelCount As Short, samplesPerSecond As Integer, bitsPerSecond As Short) As WaveFormat
        If channelCount < 1 Then
            Throw New ArgumentOutOfRangeException("channelCount")
        ElseIf samplesPerSecond < 1 Then
            Throw New ArgumentOutOfRangeException("samplesPerSecond")
        ElseIf bitsPerSecond < 1 Then
            Throw New ArgumentOutOfRangeException("bitsPerSecond")
        End If

        Dim format As New WaveFormat()

        format.FormatTag = 1
        ' ----------------------------------------------------------- PCM/Uncompressed
        format.SizeOfExtraData = 0
        format.Channels = channelCount
        format.BitsPerSample = bitsPerSecond
        format.SamplesPerSecond = samplesPerSecond

        If bitsPerSecond Mod 8 = 0 Then
            format.BlockAlign = CShort(((bitsPerSecond \ 8) * channelCount))
        Else
            format.BlockAlign = CShort(((bitsPerSecond \ 8) * channelCount + 1))
        End If

        format.AverageBytesPerSeconds = format.BlockAlign * samplesPerSecond

        Return format
    End Function

	Public ReadOnly Property CanSetPitch() As Boolean
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If
            Return (caps.support And WAVECAPS_PITCH) = WAVECAPS_PITCH
        End Get
	End Property

	Public ReadOnly Property CanSetPlaybackRate() As Boolean
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If
            Return (caps.support And WAVECAPS_PLAYBACKRATE) = WAVECAPS_PLAYBACKRATE
        End Get
	End Property

	Public ReadOnly Property CanSetVolume() As Boolean
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If
            Return (caps.support And WAVECAPS_VOLUME) = WAVECAPS_VOLUME
        End Get
	End Property

	Public ReadOnly Property CanSetStereoVolume() As Boolean
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If
            Return (caps.support And WAVECAPS_LRVOLUME) = WAVECAPS_LRVOLUME
        End Get
	End Property

	Public Property Pitch() As Integer
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If

            Dim _pitch As Integer = 0

            Dim result As Integer = NativeMethods.waveOutGetPitch(m_handle, _pitch)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If

            Return _pitch
        End Get
		Set
            If Value < 0 Then
                Throw New ArgumentOutOfRangeException("PitchMultiplier", Value, "Pitch multiplier out of range.")
            End If

            Dim result As Integer = NativeMethods.waveOutSetPitch(m_handle, Value)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If
        End Set
	End Property

	Public Property PlaybackRate() As Integer
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If

            Dim rate As Integer = 0

            Dim result As Integer = NativeMethods.waveOutGetPlaybackRate(m_handle, rate)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If

            Return rate
        End Get
		Set
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            ElseIf Value < 0 Then
                Throw New ArgumentOutOfRangeException("PlaybackRateMultiplier", Value, "Playback rate multiplier out of range.")
            End If

            Dim result As Integer = NativeMethods.waveOutSetPlaybackRate(m_handle, Value)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If
        End Set
	End Property

	Public Property Volume() As Integer
		Get
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If

            Dim _volume As Integer = 0

            Dim result As Integer = NativeMethods.waveOutGetVolume(m_handle, _volume)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If

            Return _volume
        End Get
		Set
            If IsDisposed Then
                Throw New ObjectDisposedException("OutputDevice")
            End If

            Dim result As Integer = NativeMethods.waveOutSetVolume(m_handle, Value)

            If result <> DeviceException.MMSYSERR_NOERROR Then
                Throw New OutputDeviceException(result)
            End If
        End Set
	End Property

	Public Property PollingInterval() As Double
		Get
			Return timer.Interval
		End Get
		Set
			timer.Interval = value
		End Set
	End Property

	Public Shared ReadOnly Property DeviceCount() As Integer
		Get
			Return NativeMethods.waveOutGetNumDevs()
		End Get
	End Property

    Public Overloads Overrides Sub Dispose()
        If IsDisposed Then
            Return
        End If
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Public Overloads Overrides ReadOnly Property Handle() As Int32
        Get
            Return m_handle
        End Get
    End Property

End Class



Public Class OutputDeviceException
    Inherits WaveDeviceException

    Private m_message As New StringBuilder(128)

    Public Sub New(ByVal errCode As Integer)
        MyBase.New(errCode)
        ' Get error message.
        NativeMethods.waveOutGetErrorText(errCode, m_message, m_message.Capacity)
    End Sub

    Public Overloads Overrides ReadOnly Property Message() As String
        Get
            Return m_message.ToString()
        End Get
    End Property

End Class
