

Imports System.Runtime.InteropServices

Public NotInheritable Class WaveformBuffer
	Implements IDisposable

    Public Const DefaultLength As Integer = 4096

	Private Const WHDR_DONE As Integer = &H1
	Private header As New WaveHeader()
	Private headerPtr As IntPtr
	Private buffer As Byte()
	Private bufferPtr As IntPtr

	Public Sub New()
		Initialize(DefaultLength)
	End Sub

	Public Sub New(length As Integer)
		Initialize(length)
	End Sub

	Private Sub Initialize(length As Integer)
        If length < 0 Then
            Throw New ArgumentOutOfRangeException("length")
        End If

        buffer = New Byte(length - 1) {}

        headerPtr = Marshal.AllocHGlobal(Marshal.SizeOf(GetType(WaveHeader)))

        Try
            bufferPtr = Marshal.AllocHGlobal(buffer.Length)
        Catch
            Marshal.FreeHGlobal(headerPtr)

            Throw
        End Try

        header.bufferLength = buffer.Length
        header.bytesRecorded = 0
        header.flags = 0
        header.data = bufferPtr

        Marshal.StructureToPtr(header, headerPtr, True)
    End Sub

	Public Sub Clear()
		Array.Clear(buffer, 0, buffer.Length)
	End Sub

	Public Function GetBuffer() As Byte()
		Return buffer
	End Function

	Friend Sub CopyToMemory()
		Marshal.Copy(buffer, 0, bufferPtr, buffer.Length)
	End Sub

	Public Default Property Item(index As Integer) As Byte
		Get
			Return buffer(index)
		End Get
		Set
			buffer(index) = value
		End Set
	End Property

	Friend ReadOnly Property IsDone() As Boolean
		Get
			header = CType(Marshal.PtrToStructure(headerPtr, GetType(WaveHeader)), WaveHeader)
			Return (header.flags And WHDR_DONE) = WHDR_DONE
		End Get
	End Property

	Public ReadOnly Property Length() As Integer
		Get
			Return buffer.Length
		End Get
	End Property

	Friend ReadOnly Property HeaderPointer() As IntPtr
		Get
			Return headerPtr
		End Get
	End Property

    Public Sub Dispose() Implements IDisposable.Dispose
        Marshal.FreeHGlobal(headerPtr)
        Marshal.FreeHGlobal(bufferPtr)
    End Sub

End Class
