﻿

Imports System.Threading
Imports System.Windows.Forms
Imports LC.Multimedia
Imports LC.Midi


Partial Public Class Form1
    Inherits Form

    Private Const SysExBufferSize As Integer = 128
    Private context As SynchronizationContext

    Protected Overloads Overrides Sub OnLoad(ByVal e As EventArgs)
        Text = AppTitleAndVersion("Theremino System - Midi Watcher")
        Load_INI()
        ShowInDeviceName()
        OpenInputDevice()
        context = SynchronizationContext.Current
        'startButton_Click(Nothing, Nothing)
    End Sub

    Protected Overloads Overrides Sub OnClosed(ByVal e As EventArgs)
        Me.Visible = False
        CloseInputDevice()
        Save_INI()
    End Sub

    Private Sub startButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles startButton.Click
        ListBox1.Items.Clear()
        ListBox2.Items.Clear()
        If MidiInDevice Is Nothing Then Return
        MidiInDevice.StartRecording()
    End Sub

    Private Sub stopButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles stopButton.Click
        If MidiInDevice Is Nothing Then Return
        MidiInDevice.StopRecording()
        MidiInDevice.Reset()
    End Sub


    ' ====================================================================================================
    '  INPUT MIDI DEVICE
    ' ====================================================================================================
    Friend InputDeviceID As Int32 = -1
    Private WithEvents MidiInDevice As InputDevice
    Private Function InputDeviceName(ByVal id As Int32) As String
        If id < 0 Or id >= InputDevice.DeviceCount Then Return "Unused"
        Dim cap As MidiInCaps = InputDevice.GetDeviceCapabilities(id)
        Return cap.name
    End Function
    Private Sub ShowInDeviceName()
        Combo_Init(cmb_InputDevice, InputDeviceName(InputDeviceID))
    End Sub
    Private Sub OpenInputDevice()
        CloseInputDevice()
        If InputDeviceID < 0 Or InputDeviceID >= InputDevice.DeviceCount Then Return
        MidiInDevice = New InputDevice(InputDeviceID)
        'MidiInDevice.StartRecording()
    End Sub
    Private Sub CloseInputDevice()
        If MidiInDevice IsNot Nothing Then
            MidiInDevice.StopRecording()
            MidiInDevice.Close()
            MidiInDevice = Nothing
        End If
    End Sub

    ' ====================================================================================================
    '   INPUT DEVICE COMBO
    ' ====================================================================================================
    Private Sub cmb_InputDevice_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_InputDevice.DropDown
        If InputDevice.DeviceCount > 0 Then
            Dim oldname As String = cmb_InputDevice.Items(0).ToString
            cmb_InputDevice.ItemHeight = 16
            cmb_InputDevice.Items.Clear()
            cmb_InputDevice.Items.Add("Unused")
            For i As Integer = 0 To InputDevice.DeviceCount - 1
                cmb_InputDevice.Items.Add(InputDeviceName(i))
            Next
            If InputDeviceID >= 0 And InputDeviceID < cmb_InputDevice.Items.Count - 1 Then
                cmb_InputDevice.SelectedIndex = InputDeviceID + 1
            End If
            Combo_SetIndex_FromString(cmb_InputDevice, oldname)
        End If
    End Sub
    Private Sub cmb_InputDevice_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_InputDevice.DropDownClosed
        cmb_InputDevice.ItemHeight = 12
        Combo_Init(cmb_InputDevice, cmb_InputDevice.Text)
    End Sub
    Private Sub cmb_InputDevice_SelectionChangeCommitted(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_InputDevice.SelectionChangeCommitted
        InputDeviceID = cmb_InputDevice.SelectedIndex - 1
        OpenInputDevice()
        Save_INI()
    End Sub

    ' =======================================================================================================
    '  InputDevice handlers
    ' =======================================================================================================
    'Private Sub MidiInDevice_InvalidShortMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.InvalidShortMessageEventArgs) Handles MidiInDevice.InvalidShortMessageReceived

    'End Sub
    'Private Sub MidiInDevice_InvalidSysExMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.InvalidSysExMessageEventArgs) Handles MidiInDevice.InvalidSysExMessageReceived

    'End Sub
    'Private Sub inDevice_Error(ByVal sender As Object, ByVal e As ErrorEventArgs)

    'End Sub
    'Private Sub MidiInDevice_SysExMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.SysExMessageEventArgs) Handles MidiInDevice.SysExMessageReceived
    '    Beep()
    'End Sub
    'Private Sub MidiInDevice_SysCommonMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.SysCommonMessageEventArgs) Handles MidiInDevice.SysCommonMessageReceived
    '    Beep()
    'End Sub
    'Private Sub MidiInDevice_SysRealtimeMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.SysRealtimeMessageEventArgs) Handles MidiInDevice.SysRealtimeMessageReceived
    '    Beep()
    'End Sub

    Private Sub MidiInDevice_ChannelMessageReceived(ByVal sender As Object, ByVal e As LC.Midi.ChannelMessageEventArgs) Handles MidiInDevice.ChannelMessageReceived
        context.Post(AddressOf ChannelMessageReceived, e)
    End Sub
    Private Sub ChannelMessageReceived(ByVal state As Object)
        ' ---------------------------------------------------------------------------------
        Dim args As ChannelMessageEventArgs = DirectCast(state, ChannelMessageEventArgs)
        ' ---------------------------------------------------------------------------------
        Select Case args.Message.Command

            Case ChannelCommand.NoteOn
                ListBox1.Items.Add(args.Message.Command.ToString() & "       " & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & "    " & _
                                   "Note: " & args.Message.Data1.ToString.PadLeft(3, "0"c) & "    " & _
                                   "Vel: " & args.Message.Data2.ToString)
                ListBox1.SelectedIndex = ListBox1.Items.Count - 1

            Case ChannelCommand.NoteOff
                ListBox1.Items.Add(args.Message.Command.ToString() & "       " & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & "    " & _
                                   "Note: " & args.Message.Data1.ToString.PadLeft(3, "0"c) & "    " & _
                                   "Vel: " & args.Message.Data2.ToString)
                ListBox1.SelectedIndex = ListBox1.Items.Count - 1

            Case ChannelCommand.ChannelPressure
                ListBox2.Items.Add("Channel pressure" & vbTab & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & vbTab & _
                                   "After touch: " & args.Message.Data1.ToString)
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

            Case ChannelCommand.PolyPressure
                ListBox2.Items.Add("Poly pressure" & vbTab & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & vbTab & _
                                   "Key number: " & args.Message.Data1.ToString & vbTab & _
                                   "After touch: " & args.Message.Data2.ToString)
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

            Case ChannelCommand.Controller
                ListBox2.Items.Add("Control change" & vbTab & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & vbTab & _
                                   ControlName(args.Message.Data1) & vbTab & _
                                   "Value: " & args.Message.Data2.ToString)
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

            Case ChannelCommand.ProgramChange
                ListBox2.Items.Add("Program change" & vbTab & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & vbTab & _
                                   "Program number: " & args.Message.Data1.ToString)
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

            Case ChannelCommand.PitchWheel
                ListBox2.Items.Add(args.Message.Command.ToString() & vbTab & _
                                   "Channel: " & GetChannelString(args.Message.MidiChannel) & vbTab & _
                                   "Value: " & (args.Message.Data1 + 128 * args.Message.Data2).ToString())
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

            Case Else
                ListBox2.Items.Add(args.Message.Command.ToString)
                ListBox2.SelectedIndex = ListBox2.Items.Count - 1

        End Select
    End Sub

    Private Function GetChannelString(ByVal rawChannel As Int32) As String
        Return (rawChannel + 1).ToString.PadLeft(2, "0"c)
    End Function

    Private Function ControlName(ByVal n As Int32) As String
        Select Case n
            Case 0 : Return "Bank select"
            Case 1 : Return "Modulation wheel"
            Case 2 : Return "Breath control"

            Case 4 : Return "Foot control"
            Case 5 : Return "Portamento time"
            Case 6 : Return "Data entry"
            Case 7 : Return "Channel volume"
            Case 8 : Return "Balance"

            Case 10 : Return "Pan"
            Case 11 : Return "Expression"
            Case 12 : Return "Effect 1"
            Case 13 : Return "Effect 2"

            Case 16 : Return "General 1"
            Case 17 : Return "General 2"
            Case 18 : Return "General 3"
            Case 19 : Return "General 4"

            Case 32 : Return "Bank select"
            Case 33 : Return "Mod. wheel"
            Case 34 : Return "Breath control"
            Case 36 : Return "Foot control"
            Case 37 : Return "Portamento time"
            Case 38 : Return "Data entry"
            Case 39 : Return "Channel volume"
            Case 40 : Return "Balance"

            Case 42 : Return "Pan"
            Case 43 : Return "Expression"
            Case 44 : Return "Effect 1"
            Case 45 : Return "Effect 2"

            Case 48 : Return "General 1"
            Case 49 : Return "General 2"
            Case 50 : Return "General 3"
            Case 51 : Return "General 4"

            Case 64 : Return "Dumper pedal"
            Case 65 : Return "Portamento on/off"
            Case 66 : Return "Sustain on/off"
            Case 67 : Return "Soft pedal on/off"
            Case 68 : Return "Legato switch"
            Case 69 : Return "Hold 2"

            Case 70, 71, 72, 73, 74, 75, 76, 77, 78, 79 : Return "Sound controller(" & (n - 69).ToString & ")"

            Case 80 : Return "General 5"
            Case 81 : Return "General 6"
            Case 82 : Return "General 7"
            Case 83 : Return "General 8"
            Case 84 : Return "Portamento"

            Case 91 : Return "Effect 1 depth"
            Case 92 : Return "Effect 2 depth"
            Case 93 : Return "Effect 3 depth"
            Case 94 : Return "Effect 4 depth"
            Case 95 : Return "Effect 5 depth"

            Case 96 : Return "Data entry +1"
            Case 97 : Return "Data entry -1"

            Case 98 : Return "NRPN LSB"
            Case 99 : Return "NRPN MSB"
            Case 100 : Return "RPN LSB"
            Case 101 : Return "RPN MSB"

            Case 120 : Return "All sound off"
            Case 121 : Return "Reset all contr."
            Case 122 : Return "Local cntr. on/off"
            Case 123 : Return "All note off"
            Case 124 : Return "Omni mode off"
            Case 125 : Return "Omni mode on"
            Case 126 : Return "Poly mode on/off"
            Case 127 : Return "Poly mode on"

            Case Else : Return "Undefined(" & n.ToString & ")"
        End Select
    End Function


End Class