﻿Imports System.Drawing.Drawing2D

Module Module_FillBars

    ' =======================================================================================
    '  FILL SLIDER VOL & PITCH
    ' =======================================================================================
    Friend Sub FillSliderIndicator(ByVal pbox As PictureBox, _
                                   ByVal value As Double, _
                                   ByVal c1 As Color, _
                                   ByVal c2 As Color)
        ' ----------------------------------------------------------- 
        If pbox Is Nothing Then Exit Sub
        If Not pbox.Visible Then Exit Sub
        ' -------------------------------------------------------
        Dim w As Int32 = pbox.ClientSize.Width
        Dim h As Int32 = pbox.ClientSize.Height
        ' ------------------------------------------------------- using a bitmap for best performance
        If pbox.Image Is Nothing Then
            pbox.Image = New Bitmap(w, h)
        End If
        ' -------------------------------------------------------
        If value < 0 Then value = 0
        If value > 1 Then value = 1
        Dim v As Int32 = CInt(value * w)
        ' ------------------------------------------------------- using "Graphics.FromImage" for best performance
        Dim g As Graphics = Graphics.FromImage(pbox.Image)
        g.FillRectangle(New SolidBrush(c1), 0, 0, v, h)
        g.FillRectangle(New SolidBrush(c2), v, 0, w - v, h)
        ' ------------------------------------------------------- final refresh - no flickering
        pbox.Invalidate()
    End Sub

End Module


Friend Class VuMeter

    Private pbox As PictureBox
    Private w As Int32
    Private h As Int32
    Private b1 As Drawing2D.LinearGradientBrush
    Private b2 As Brush
    Private g As Graphics
    Private oldvalue As Double

    Friend Sub New(ByVal _pbox As PictureBox)
        pbox = _pbox
    End Sub

    Friend Sub Draw(ByVal value As Double, Optional ByVal MarkPoint As Double = 0)
        ' ----------------------------------------------------------- 
        If pbox Is Nothing Then Exit Sub
        If Not pbox.Visible Then Exit Sub
        ' ----------------------------------------------------------- 
        If value < 0.0001 Then value = 0
        If value > 1 Then value = 1
        If value = oldvalue Then Return
        oldvalue = value
        ' ----------------------------------------------------------- 
        If pbox.Image Is Nothing OrElse pbox.ClientSize.Width <> w OrElse pbox.ClientSize.Height <> h Then
            w = pbox.ClientSize.Width
            h = pbox.ClientSize.Height
            pbox.Image = New Bitmap(w, h)
            g = Graphics.FromImage(pbox.Image)
            b1 = CreateFillBrush(pbox, If(w > h, LinearGradientMode.Horizontal, LinearGradientMode.Vertical))
            b2 = New SolidBrush(pbox.BackColor)
        End If
        ' ----------------------------------------------------------- 
        If w > h Then
            Dim v As Int32 = CInt(value * w)
            g.FillRectangle(b1, 0, 0, v, h)
            g.FillRectangle(b2, v, 0, w, h)
            ' ----------------------------------------------------------- mark line
            If MarkPoint > 0 Then
                v = CInt(MarkPoint * w)
                g.DrawLine(Pens.Black, v, 0, v, h)
            End If
        Else
            Dim v As Int32 = CInt(value * h)
            g.FillRectangle(b1, 0, h - v, w, h)
            g.FillRectangle(b2, 0, 0, w, h - v)
            ' ----------------------------------------------------------- mark line
            If MarkPoint > 0 Then
                v = CInt(MarkPoint * h)
                g.DrawLine(Pens.Black, 0, h - v, w, h - v)
            End If
        End If

        Dim dbStr As String = ""
        If value > 0.001 Then
            dbStr = VoltToDecibel(value).ToString("0") & " dB"
        End If
        ' ----------------------------------------------------------- 
        Dim f As Font
        f = New Font("Tahoma", pbox.ClientSize.Height - 2, FontStyle.Regular, GraphicsUnit.Pixel)
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
        ' -----------------------------------------------------------
        Dim stringFormat As New StringFormat()
        stringFormat.Alignment = StringAlignment.Far
        stringFormat.LineAlignment = StringAlignment.Center
        Dim rec As New Rectangle(0, 0, CInt(w / 2 + 17), h)
        g.DrawString(dbStr, f, Brushes.Black, rec, stringFormat)
        ' ----------------------------------------------------------- final refresh - no flickering
        pbox.Invalidate()
    End Sub

    Private Function CreateFillBrush(ByVal pbox As PictureBox, _
                             ByVal direction As LinearGradientMode) As LinearGradientBrush
        CreateFillBrush = New LinearGradientBrush(pbox.ClientRectangle, _
                                                  Color.Black, _
                                                  Color.Black, _
                                                  direction)
        Dim myBlend As ColorBlend = New ColorBlend()
        If direction = LinearGradientMode.Vertical Then
            myBlend.Colors = New Color() {Color.FromArgb(255, 0, 0), _
                                          Color.FromArgb(255, 200, 0), _
                                          Color.FromArgb(100, 255, 0)}
        Else
            myBlend.Colors = New Color() {Color.FromArgb(100, 255, 0), _
                                          Color.FromArgb(255, 200, 0), _
                                          Color.FromArgb(255, 0, 0)}
        End If
        myBlend.Positions = New Single() {0.0F, 0.5F, 1.0F}
        CreateFillBrush.InterpolationColors = myBlend
    End Function

    Private Function VoltToDecibel(ByRef v As Double) As Double
        VoltToDecibel = 20 * Math.Log(v, 10)
    End Function

End Class


Friend Class TunerMeter

    Private pbox As PictureBox
    Private w As Int32
    Private h As Int32
    Private b1 As Drawing2D.LinearGradientBrush
    Private b2 As Brush
    Private g As Graphics
    Private oldvalue As Double

    Friend Sub New(ByVal _pbox As PictureBox)
        pbox = _pbox
    End Sub

    Friend Sub Draw(ByVal value As Double)
        ' ----------------------------------------------------------- 
        If pbox Is Nothing Then Exit Sub
        If Not pbox.Visible Then Exit Sub
        ' ----------------------------------------------------------- 
        If value < 0 Then value = 0
        If value > 1 Then value = 1
        If value = oldvalue Then Return
        oldvalue = value
        ' ----------------------------------------------------------- 
        If pbox.Image Is Nothing OrElse pbox.ClientSize.Width <> w OrElse pbox.ClientSize.Height <> h Then
            w = pbox.ClientSize.Width
            h = pbox.ClientSize.Height
            pbox.Image = New Bitmap(w, h)
            g = Graphics.FromImage(pbox.Image)
            b1 = CreateFillBrush(pbox)
            b2 = New SolidBrush(pbox.BackColor)
        End If
        ' ----------------------------------------------------------- 
        Dim v As Int32 = CInt(value * w)
        Dim vmid As Int32 = CInt(w / 2.0F)
        If v < vmid Then
            g.FillRectangle(b2, 0, 0, v, h)
            g.FillRectangle(b1, v, 0, vmid, h)
            g.FillRectangle(b2, vmid, 0, w, h)
        Else
            g.FillRectangle(b2, 0, 0, vmid, h)
            g.FillRectangle(b1, vmid, 0, v, h)
            g.FillRectangle(b2, v, 0, w, h)
        End If
        '' ----------------------------------------------------------- 
        'Dim StrText As String
        'StrText = (value * 100 - 50).ToString("0")
        '' ----------------------------------------------------------- 
        'Dim f As Font
        'f = New Font("Tahoma", h - 2, FontStyle.Regular, GraphicsUnit.Pixel)
        'g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
        '' ----------------------------------------------------------- 
        'Dim stringFormat As New StringFormat()
        'stringFormat.Alignment = StringAlignment.Far
        'stringFormat.LineAlignment = StringAlignment.Center
        'Dim rec As New Rectangle(0, 0, CInt(w / 2 + 17), h )
        'g.DrawString(StrText, f, Brushes.Black, rec, stringFormat)
        ' ----------------------------------------------------------- final refresh - no flickering
        pbox.Invalidate()
    End Sub

    Private Function CreateFillBrush(ByVal pbox As PictureBox) As LinearGradientBrush
        CreateFillBrush = New LinearGradientBrush(pbox.ClientRectangle, _
                                                  Color.Black, _
                                                  Color.Black, _
                                                  LinearGradientMode.Horizontal)
        Dim myBlend As ColorBlend = New ColorBlend()
        myBlend.Colors = New Color() {Color.FromArgb(255, 0, 0), _
                                      Color.FromArgb(255, 200, 0), _
                                      Color.FromArgb(100, 255, 0), _
                                      Color.FromArgb(255, 200, 0), _
                                      Color.FromArgb(255, 0, 0)}
        myBlend.Positions = New Single() {0.0F, 0.35F, 0.5F, 0.65F, 1.0F}
        CreateFillBrush.InterpolationColors = myBlend
    End Function

End Class







