

Imports LC.Synth


Public Class SynthVB_Voice
    Inherits Voice

    ' ---------------------------------------------------- for use as a portamento source
    Private slewLimiter As SlewLimiter
    ' ---------------------------------------------------- two oscillators ( typical of most subtractive synths )
    Private osc1 As Oscillator
    Private osc2 As Oscillator
    ' ---------------------------------------------------- LFO's to use as modulation sources
    Private lfo1 As Lfo
    Private lfo2 As Lfo
    ' ---------------------------------------------------- ADSR envelopes to use as modulation source
    '                                                      first envelope is hardwired to modulate the overall amplitude
    Private envelope1 As AdsrEnvelope
    Private envelope2 As AdsrEnvelope
    '  --------------------------------------------------- for filtering the sound
    Private filter As StateVariableFilter
    '  --------------------------------------------------- convert the mono output of the filter into stereo
    Private converter As MonoToStereoConverter
    '  --------------------------------------------------- for use when no modulation source is set
    Private emptyFMModulator As EmptyMonoComponent
    '  --------------------------------------------------- for choosing which waveform the oscillators use
    Private oscWaveProgrammer1 As OscWaveProgrammer
    Private oscWaveProgrammer2 As OscWaveProgrammer
    '  --------------------------------------------------- for choosing the oscillators' FM source
    Private oscFMProgrammer1 As OscFMProgrammer
    Private oscFMProgrammer2 As OscFMProgrammer


    Public Sub New(ByVal sampleRate As SampleRate, ByVal buffer As StereoBuffer)
        MyBase.New(sampleRate, buffer)
        Initialize(sampleRate, buffer)
    End Sub

    Public Sub New(ByVal sampleRate As SampleRate, ByVal buffer As StereoBuffer, ByVal name As String)
        MyBase.New(sampleRate, buffer, name)
        Initialize(sampleRate, buffer)
    End Sub


    Private Sub Initialize(ByVal sampleRate As SampleRate, ByVal buffer As StereoBuffer)
        emptyFMModulator = New EmptyMonoComponent(sampleRate, New MonoBuffer(0))

        slewLimiter = New SlewLimiter(sampleRate, New MonoBuffer(0), "Portamento")
        slewLimiter.SynthesizeReplaceEnabled = True

        envelope1 = New AdsrEnvelope(sampleRate, New MonoBuffer(0), "Envelope 1")
        envelope1.SynthesizeReplaceEnabled = True

        envelope2 = New AdsrEnvelope(sampleRate, New MonoBuffer(0), "Envelope 2")
        envelope2.SynthesizeReplaceEnabled = True

        lfo1 = New Lfo(sampleRate, New MonoBuffer(0), "LFO 1")
        lfo1.SynthesizeReplaceEnabled = True

        lfo2 = New Lfo(sampleRate, New MonoBuffer(0), "LFO 2")
        lfo2.SynthesizeReplaceEnabled = True

        Dim wave As Wavetable = Wavetable.Load("Sawtooth")

        osc1 = New Oscillator(sampleRate, New MonoBuffer(0), "Oscillator 1", wave, emptyFMModulator, slewLimiter)
        osc1.SynthesizeReplaceEnabled = True

        osc2 = New Oscillator(sampleRate, New MonoBuffer(0), "Oscillator 2", wave, emptyFMModulator, slewLimiter)
        osc2.SynthesizeReplaceEnabled = True

        filter = New StateVariableFilter(sampleRate, New MonoBuffer(0), "State Variable Filter", osc1, osc2, envelope2, _
         lfo2, envelope1)
        filter.SynthesizeReplaceEnabled = True

        converter = New MonoToStereoConverter(sampleRate, buffer, filter)
        converter.SynthesizeReplaceEnabled = False

        oscWaveProgrammer1 = New OscWaveProgrammer("Oscillator 1", osc1)
        oscWaveProgrammer2 = New OscWaveProgrammer("Oscillator 2", osc2)

        oscFMProgrammer1 = New OscFMProgrammer("Oscillator 1", osc1, emptyFMModulator, envelope1, envelope2, lfo1, _
         lfo2)
        oscFMProgrammer2 = New OscFMProgrammer("Oscillator 2", osc2, emptyFMModulator, envelope1, envelope2, lfo1, _
         lfo2)

        AddComponent(emptyFMModulator)
        AddComponent(slewLimiter)
        AddComponent(osc1)
        AddComponent(osc2)
        AddComponent(envelope1)
        AddComponent(envelope2)
        AddComponent(lfo1)
        AddComponent(lfo2)
        AddComponent(filter)
        AddComponent(converter)

        AddParameters(slewLimiter)
        AddParameters(osc1)
        AddParameters(oscWaveProgrammer1)
        AddParameters(oscFMProgrammer1)
        AddParameters(osc2)
        AddParameters(oscWaveProgrammer2)
        AddParameters(oscFMProgrammer2)
        AddParameters(envelope1)
        AddParameters(envelope2)
        AddParameters(lfo1)
        AddParameters(lfo2)
        AddParameters(filter)

        AddBendable(osc1)
        AddBendable(osc2)

        AddControllable(lfo1)
        AddControllable(lfo2)
    End Sub

    Protected Overloads Overrides ReadOnly Property IsPlaying() As Boolean
        Get
            Return Not envelope1.HasCompleted
        End Get
    End Property

    Public Overloads Overrides Property SynthesizeReplaceEnabled() As Boolean
        Get
            Return converter.SynthesizeReplaceEnabled
        End Get
        Set(ByVal value As Boolean)
            converter.SynthesizeReplaceEnabled = value
        End Set
    End Property

End Class
